@@ -23,6 +23,7 @@ import matplotlib.pyplot as plt
 from typing import *
 from PIL import Image
 from functools import reduce
+from itertools import product
 from collections import Counter
 from abc import abstractmethod
 
@@ -34,12 +35,12 @@ dill._dill._reverse_typemap["ClassType"] = type
 def timestamp(simplify: bool = False,
               ensure_different: bool = False) -> str:
     """
-    Return current timestamp
+    Return current timestamp.
 
     Parameters
     ----------
-    simplify : bool. If True, format will be simplified to 'year-month-day'
-    ensure_different : bool. If True, format will include millisecond
+    simplify : bool. If True, format will be simplified to 'year-month-day'.
+    ensure_different : bool. If True, format will include millisecond.
 
     Returns
     -------
@@ -56,20 +57,20 @@ def timestamp(simplify: bool = False,
 
 
 def prod(iterable: Iterable) -> float:
-    """ return cumulative production of an iterable """
+    """ Return cumulative production of an iterable. """
 
     return float(reduce(operator.mul, iterable, 1))
 
 
 def hash_code(code: str) -> str:
-    """ return hash code for a string """
+    """ Return hash code for a string. """
 
     code = code.encode()
     return hashlib.md5(code).hexdigest()[:8]
 
 
 def prefix_dict(d: Dict[str, Any], prefix: str):
-    """ prefix every key in dict `d` with `prefix` """
+    """ Prefix every key in dict `d` with `prefix`. """
 
     return {f"{prefix}_{k}": v for k, v in d.items()}
 
@@ -84,8 +85,8 @@ def shallow_copy_dict(d: dict) -> dict:
 
 def update_dict(src_dict: dict, tgt_dict: dict) -> dict:
     """
-    Update tgt_dict with src_dict
-    * Notice that changes will happen only on keys which src_dict holds
+    Update tgt_dict with src_dict.
+    * Notice that changes will happen only on keys which src_dict holds.
 
     Parameters
     ----------
@@ -110,7 +111,7 @@ def update_dict(src_dict: dict, tgt_dict: dict) -> dict:
 
 
 def fix_float_to_length(num: float, length: int) -> str:
-    """ change a float number to string format with fixed length """
+    """ Change a float number to string format with fixed length. """
 
     str_num = f"{num:f}"
     if str_num == "nan":
@@ -120,7 +121,7 @@ def fix_float_to_length(num: float, length: int) -> str:
 
 
 def truncate_string_to_length(string: str, length: int) -> str:
-    """ truncate a string to make sure its length not exceeding a given length """
+    """ Truncate a string to make sure its length not exceeding a given length. """
 
     if len(string) <= length:
         return string
@@ -129,7 +130,7 @@ def truncate_string_to_length(string: str, length: int) -> str:
 
 
 def grouped(iterable: Iterable, n: int, *, keep_tail=False) -> List[tuple]:
-    """ group an iterable every `n` elements """
+    """ Group an iterable every `n` elements. """
 
     if not keep_tail:
         return list(zip(*[iter(iterable)] * n))
@@ -138,7 +139,7 @@ def grouped(iterable: Iterable, n: int, *, keep_tail=False) -> List[tuple]:
 
 
 def is_numeric(s: str) -> bool:
-    """ check whether `s` is a number """
+    """ Check whether `s` is a number. """
 
     try:
         s = float(s)
@@ -153,12 +154,12 @@ def is_numeric(s: str) -> bool:
 
 def get_one_hot(feature: Union[list, np.ndarray], dim: int) -> np.ndarray:
     """
-    Get one-hot representation
+    Get one-hot representation.
 
     Parameters
     ----------
-    feature : array-like, source data of one-hot representation
-    dim : int, dimension of the one-hot representation
+    feature : array-like, source data of one-hot representation.
+    dim : int, dimension of the one-hot representation.
 
     Returns
     -------
@@ -174,16 +175,16 @@ def get_one_hot(feature: Union[list, np.ndarray], dim: int) -> np.ndarray:
 def show_or_save(export_path: str,
                  fig: plt.figure = None, **kwargs) -> None:
     """
-    Utility function to deal with figure
+    Utility function to deal with figure.
 
     Parameters
     ----------
     export_path : {None, str}
-        * None : the figure will be shown
-        * str  : it represents the path where the figure should be saved to
+    * If None, the figure will be shown.
+    * If str, it represents the path where the figure should be saved to.
     fig : {None, plt.Figure}
-        * None       : default figure contained in plt will be executed
-        * plt.Figure : it will be executed
+    * If None, default figure contained in plt will be executed.
+    * If plt.figure, it will be executed
 
     """
 
@@ -196,11 +197,11 @@ def show_or_save(export_path: str,
 
 def show_or_return(return_canvas: bool) -> Union[None, np.ndarray]:
     """
-    Utility function to deal with current plt
+    Utility function to deal with current plt.
 
     Parameters
     ----------
-    return_canvas : bool, whether return canvas or not
+    return_canvas : bool, whether return canvas or not.
 
     """
 
@@ -221,16 +222,16 @@ def show_or_return(return_canvas: bool) -> Union[None, np.ndarray]:
 def get_indices_from_another(base: np.ndarray,
                              segment: np.ndarray) -> np.ndarray:
     """
-    Get `segment` elements' indices in `base`
+    Get `segment` elements' indices in `base`.
 
     Warnings
     ----------
-    All elements in segment should appear in base to ensure validity
+    All elements in segment should appear in base to ensure validity.
 
     Parameters
     ----------
-    base : np.ndarray, base array
-    segment : np.ndarray, segment array
+    base : np.ndarray, base array.
+    segment : np.ndarray, segment array.
 
     Returns
     -------
@@ -250,11 +251,11 @@ def get_indices_from_another(base: np.ndarray,
 
 class UniqueIndices(NamedTuple):
     """
-    unique           : np.ndarray, unique values of the given array (`arr`)
-    unique_cnt       : np.ndarray, counts of each unique value
-    sorting_indices  : np.ndarray, indices which can (stably) sort the given array by its value
-    split_arr        : np.ndarray, array which can split the `sorting_indices` to make sure that
-                       each portion of the split indices belong & only belong to one of the unique values
+    unique           : np.ndarray, unique values of the given array (`arr`).
+    unique_cnt       : np.ndarray, counts of each unique value.
+    sorting_indices  : np.ndarray, indices which can (stably) sort the given array by its value.
+    split_arr        : np.ndarray, array which can split the `sorting_indices` to make sure that.
+                       each portion of the split indices belong & only belong to one of the unique values.
     """
     unique: np.ndarray
     unique_cnt: np.ndarray
@@ -268,11 +269,11 @@ class UniqueIndices(NamedTuple):
 
 def get_unique_indices(arr: np.ndarray) -> UniqueIndices:
     """
-    Get indices for unique values of an array
+    Get indices for unique values of an array.
 
     Parameters
     ----------
-    arr : np.ndarray, target array which we wish to find indices of each unique value
+    arr : np.ndarray, target array which we wish to find indices of each unique value.
 
     Returns
     -------
@@ -299,11 +300,11 @@ def get_unique_indices(arr: np.ndarray) -> UniqueIndices:
 
 def get_counter_from_arr(arr: np.ndarray) -> Counter:
     """
-    Get `Counter` of an array
+    Get `Counter` of an array.
 
     Parameters
     ----------
-    arr : np.ndarray, target array which we wish to get `Counter` from
+    arr : np.ndarray, target array which we wish to get `Counter` from.
 
     Returns
     -------
@@ -324,12 +325,12 @@ def get_counter_from_arr(arr: np.ndarray) -> Counter:
 
 def allclose(*arrays: np.ndarray, **kwargs) -> bool:
     """
-    Perform `np.allclose` to `arrays` one by one
+    Perform `np.allclose` to `arrays` one by one.
 
     Parameters
     ----------
-    arrays : np.ndarray, target arrays
-    **kwargs : keyword arguments which will be passed into `np.allclose`
+    arrays : np.ndarray, target arrays.
+    **kwargs : keyword arguments which will be passed into `np.allclose`.
 
     Returns
     -------
@@ -365,12 +366,12 @@ def register_core(name: str,
 
 class Incrementer:
     """
-    Util class which can calculate running mean & running std efficiently
+    Util class which can calculate running mean & running std efficiently.
 
     Parameters
     ----------
-    window_size : {int, None}, window size of running statistics
-        * None : then all history records will be used for calculation
+    window_size : {int, None}, window size of running statistics.
+    * If None, then all history records will be used for calculation.
 
     Examples
     ----------
@@ -423,7 +424,7 @@ class Incrementer:
 
 
 class _Formatter(logging.Formatter):
-    """ formatter for logging, which supports millisecond """
+    """ Formatter for logging, which supports millisecond. """
 
     converter = datetime.datetime.fromtimestamp
 
@@ -439,29 +440,29 @@ class _Formatter(logging.Formatter):
 
 class LoggingMixin:
     """
-    Mixin class to provide logging methods for base class
+    Mixin class to provide logging methods for base class.
 
     Attributes
     ----------
     _triggered_ : bool
-        If not `_triggered_`, log file will not be created
+    * If not `_triggered_`, log file will not be created.
 
     _verbose_level_ : int
-        Preset verbose level of the whole logging process
+    * Preset verbose level of the whole logging process.
 
     Methods
     ----------
     log_msg(self, body, prefix="", verbose_level=1)
-        Log something either through console or to a file
+        Log something either through console or to a file.
         * body : str
-            main logging message
+            Main logging message.
         * prefix : str
-            prefix added to `body` when logging message goes through console
+            Prefix added to `body` when logging message goes through console.
         * verbose_level : int
-            if `self._verbose_level_` >= verbose_level, then logging message will go through console
+            If `self._verbose_level_` >= verbose_level, then logging message will go through console.
 
     log_block_msg(self, body, prefix="", title="", verbose_level=1)
-        Almost the same as `log_msg`, except adding `title` on top of `body`
+        Almost the same as `log_msg`, except adding `title` on top of `body`.
 
     """
 
@@ -654,23 +655,23 @@ class LoggingMixin:
 
 class PureLoggingMixin:
     """
-    Mixin class to provide (pure) logging method for base class
+    Mixin class to provide (pure) logging method for base class.
 
     Attributes
     ----------
     _loggers_ : dict(int, logging.Logger)
-        Recorded all loggers initialized
+        Recorded all loggers initialized.
 
     _formatter_ : _Formatter
-        Formatter for all loggers
+        Formatter for all loggers.
 
     Methods
     ----------
     log_msg(self, name, msg, msg_level=logging.INFO)
-        Log something to a file, with logger initialized by `name`
+        Log something to a file, with logger initialized by `name`.
 
     log_block_msg(self, name, title, body, msg_level=logging.INFO)
-        Almost the same as `log_msg`, except adding `title` on top of `body`
+        Almost the same as `log_msg`, except adding `title` on top of `body`.
 
     """
 
@@ -769,15 +770,15 @@ class PureLoggingMixin:
 
 class SavingMixin(LoggingMixin):
     """
-    Mixin class to provide logging & saving method for base class
+    Mixin class to provide logging & saving method for base class.
 
     Methods
     ----------
     save(self, folder)
-        Save `self` to folder
+        Save `self` to folder.
 
     def load(self, folder)
-        Load from folder
+        Load from folder.
 
     """
 
@@ -826,23 +827,23 @@ class SavingMixin(LoggingMixin):
 
 class Saving(LoggingMixin):
     """
-    Util class for saving instances
+    Util class for saving instances.
 
     Methods
     ----------
     save_instance(instance, folder, log_method=None)
-        Save instance to `folder`
-        * instance : object, instance to save
-        * folder : str, folder to save to
+        Save instance to `folder`.
+        * instance : object, instance to save.
+        * folder : str, folder to save to.
         * log_method : {None, function}, used as `log_method` parameter in
-        `log_with_external_method` method of `LoggingMixin`
+        `log_with_external_method` method of `LoggingMixin`.
 
     load_instance(instance, folder, log_method=None)
-        Load instance from `folder`
-        * instance : object, instance to load, need to be initialized
-        * folder : str, folder to load from
+        Load instance from `folder`.
+        * instance : object, instance to load, need to be initialized.
+        * folder : str, folder to load from.
         * log_method : {None, function}, used as `log_method` parameter in
-        `log_with_external_method` method of `LoggingMixin`
+        `log_with_external_method` method of `LoggingMixin`.
 
     """
 
@@ -1062,15 +1063,185 @@ class Saving(LoggingMixin):
         return _manager()
 
 
-class Sampler:
+candidate_type = List[Any]
+candidates_type = Union[List[candidate_type], Dict[str, candidate_type]]
+
+
+class Grid:
     """
-    Util class which can help sampling indices from probabilities
+    Util class provides permutation of simple, flattened candidates.
+    * For permutation of complex, nested param dicts, please refers to `ParamGenerator` in `cftool.param_utils.core`.
 
     Parameters
     ----------
-    method : str, sampling method
-    * currently only 'multinomial' is supported
-    probabilities : np.ndarray, probabilities we'll use
+    candidates : candidates_type, cadidates we want to create grid from.
+
+    Examples
+    ----------
+    >>> from cftool.misc import Grid
+    >>>
+    >>> grid = Grid({"a": [1, 2, 3], "b": [1, 2, 3]})
+    >>> for param in grid:
+    >>>     print(param)
+    >>> # output : {'a': 1, 'b': 1}, {'a': 1, 'b': 2}, {'a': 1, 'b': 3}
+    >>> #          {'a': 2, 'b': 1}, ..., {'a': 3, 'b': 3}
+
+    """
+
+    def __init__(self, candidates: candidates_type):
+        self.candidates = candidates
+        self._is_list = isinstance(candidates, list)
+
+    @staticmethod
+    def _yield_lists(lists):
+        yield from map(list, product(*lists))
+
+    def __iter__(self):
+        if self._is_list:
+            yield from self._yield_lists(self.candidates)
+        else:
+            items = sorted(self.candidates.items())
+            if not items:
+                yield {}
+            else:
+                keys, values = zip(*items)
+                for v in map(list, product(*values)):
+                    yield dict(zip(keys, v))
+
+
+nested_type = Dict[str, Union[Any, Dict[str, "nested_type"]]]
+all_nested_type = Dict[str, Union[List[Any], Dict[str, "all_nested_type"]]]
+union_nested_type = Union[nested_type, all_nested_type]
+flattened_type = Dict[str, Any]
+all_flattened_type = Dict[str, List[Any]]
+union_flattened_type = Union[flattened_type, all_flattened_type]
+
+
+def _offset_fn(value) -> int:
+    if not isinstance(value, (list, tuple)):
+        return 1
+    return len(value)
+
+
+class Nested:
+    def __init__(self,
+                 nested: union_nested_type,
+                 *,
+                 offset_fn: Callable[[Any], int] = _offset_fn,
+                 delim: str = "^_^"):
+        self.nested = nested
+        self.offset_fn, self.delim = offset_fn, delim
+        self._flattened = self._sorted_flattened_keys = None
+        self._sorted_flattened_offsets = None
+
+    def apply(self,
+              fn: Callable[[Any], Any]) -> "Nested":
+        def _apply(src, tgt):
+            for k, v in src.items():
+                if isinstance(v, dict):
+                    next_tgt = tgt.setdefault(k, {})
+                    _apply(v, next_tgt)
+                else:
+                    tgt[k] = fn(v)
+            return tgt
+        return Nested(_apply(self.nested, {}))
+
+    @property
+    def flattened(self) -> union_flattened_type:
+        if self._flattened is None:
+            self._flattened = self.flatten_nested(self.nested)
+        return self._flattened
+
+    @property
+    def sorted_flattened_keys(self) -> List[str]:
+        if self._sorted_flattened_keys is None:
+            self._sorted_flattened_keys = sorted(self.flattened)
+        return self._sorted_flattened_keys
+
+    @property
+    def sorted_flattened_offsets(self) -> List[int]:
+        if self._sorted_flattened_offsets is None:
+            offsets = []
+            for key in self.sorted_flattened_keys:
+                value = self.get_value_from(key)
+                offsets.append(self.offset_fn(value))
+            self._sorted_flattened_offsets = offsets
+        return self._sorted_flattened_offsets
+
+    def get_value_from(self, flattened_key: str) -> Any:
+        value = self.nested
+        for sub_key in flattened_key.split(self.delim):
+            value = value[sub_key]
+        return value
+
+    def flatten_nested(self,
+                       nested: nested_type) -> nested_type:
+        flattened = []
+        def _flatten(d, pre_key: Union[None, str]):
+            for name, value in d.items():
+                if pre_key is None:
+                    next_pre_key = name
+                else:
+                    next_pre_key = f"{pre_key}{self.delim}{name}"
+                if isinstance(value, dict):
+                    _flatten(value, next_pre_key)
+                else:
+                    flattened.append((next_pre_key, value))
+            return flattened
+        return dict(_flatten(nested, None))
+
+    def nest_flattened(self,
+                       flattened: flattened_type) -> nested_type:
+        sorted_pairs = sorted(map(
+            lambda k, v: (k.split(self.delim), v),
+            *zip(*flattened.items())
+        ), key=len)
+        nested = {}
+        for key_list, value in sorted_pairs:
+            if len(key_list) == 1:
+                nested[key_list[0]] = value
+            else:
+                parent = nested.setdefault(key_list[0], {})
+                for key in key_list[1:-1]:
+                    parent = parent.setdefault(key, {})
+                parent[key_list[-1]] = value
+        return nested
+
+    def flattened2array(self,
+                        flattened: flattened_type) -> np.ndarray:
+        value_list = []
+        for key in self.sorted_flattened_keys:
+            value = flattened[key]
+            value = list(value) if isinstance(value, (list, tuple)) else [value]
+            value_list.extend(value)
+        return np.array(value_list, np.float32)
+
+    def array2flattened(self,
+                        array: np.ndarray) -> flattened_type:
+        cursor = 0
+        flattened = {}
+        for key, offset in zip(self.sorted_flattened_keys, self.sorted_flattened_offsets):
+            end = cursor + offset
+            if offset == 1:
+                value = array[cursor]
+            else:
+                value = array[cursor:end].tolist()
+                if isinstance(value, tuple):
+                    value = tuple(value)
+            flattened[key] = value
+            cursor = end
+        return flattened
+
+
+class Sampler:
+    """
+    Util class which can help sampling indices from probabilities.
+
+    Parameters
+    ----------
+    method : str, sampling method.
+    * Currently only 'multinomial' is supported.
+    probabilities : np.ndarray, probabilities we'll use.
 
     Examples
     ----------
@@ -1120,7 +1291,7 @@ class Sampler:
 # contexts
 
 class context_error_handler:
-    """ Util class which provides exception handling when using context manager """
+    """ Util class which provides exception handling when using context manager. """
 
     @property
     def exception_suffix(self):
@@ -1141,7 +1312,7 @@ class context_error_handler:
 
 class timeit(context_error_handler):
     """
-    Timing context manager
+    Timing context manager.
 
     Examples
     --------
@@ -1191,7 +1362,7 @@ class _lock_file_refresher(threading.Thread):
 
 class lock_manager(context_error_handler, LoggingMixin):
     """
-    Util class to make simultaneously-write process safe with some hacked (ugly) tricks
+    Util class to make simultaneously-write process safe with some hacked (ugly) tricks.
 
     Examples
     --------
@@ -1332,13 +1503,13 @@ class lock_manager(context_error_handler, LoggingMixin):
 
 class batch_manager(context_error_handler):
     """
-    Process data in batch
+    Process data in batch.
 
     Parameters
     ----------
     inputs : tuple(np.ndarray), auxiliary array inputs.
-    n_elem : {int, float}, indicates how many elements will be processed in a batch
-    batch_size : int, indicates the batch_size; if None, batch_size will be calculated by n_elem
+    n_elem : {int, float}, indicates how many elements will be processed in a batch.
+    batch_size : int, indicates the batch_size; if None, batch_size will be calculated by n_elem.
 
     Examples
     --------
@@ -1390,12 +1561,12 @@ class batch_manager(context_error_handler):
 
 class timing_context(context_error_handler):
     """
-    Wrap codes in any base class of `LoggingMixin` with this timing context to timeit
+    Wrap codes in any base class of `LoggingMixin` with this timing context to timeit.
 
     Parameters
     ----------
-    logging_mixin : LoggingMixin, arbitrary base classes of LoggingMixin
-    name : str, explain what the wrapped codes are doing
+    logging_mixin : LoggingMixin, arbitrary base classes of LoggingMixin.
+    name : str, explain what the wrapped codes are doing.
 
     Examples
     --------
@@ -1428,12 +1599,12 @@ class timing_context(context_error_handler):
 
 class data_tuple_saving_controller(context_error_handler):
     """
-    Help saving DataTuple of SavingMixin
+    Help saving DataTuple of SavingMixin.
 
     Parameters
     ----------
-    instance : SavingMixin, instance whose DataTuples are to be saved / loaded
-    is_saving : bool, whether it is a saving context or not
+    instance : SavingMixin, instance whose DataTuples are to be saved / loaded.
+    is_saving : bool, whether it is a saving context or not.
     """
 
     __prefix__ = "_Data_Tuple__"
@@ -1504,9 +1675,11 @@ class data_tuple_saving_controller(context_error_handler):
 
 
 __all__ = [
-    "get_indices_from_another", "get_unique_indices", "get_one_hot", "hash_code", "prefix_dict",
-    "timestamp", "fix_float_to_length", "truncate_string_to_length", "grouped", "is_numeric", "update_dict",
-    "show_or_save", "show_or_return", "Saving", "LoggingMixin", "PureLoggingMixin", "SavingMixin", "Sampler",
-    "context_error_handler", "timeit", "lock_manager", "batch_manager", "timing_context", "prod",
-    "get_counter_from_arr", "allclose", "shallow_copy_dict", "register_core", "data_tuple_saving_controller"
+    "timestamp", "prod", "hash_code", "prefix_dict", "shallow_copy_dict", "update_dict", "fix_float_to_length",
+    "truncate_string_to_length", "grouped", "is_numeric", "get_one_hot", "show_or_save", "show_or_return",
+    "get_indices_from_another", "UniqueIndices", "get_unique_indices", "get_counter_from_arr", "allclose",
+    "register_core", "Incrementer", "LoggingMixin", "PureLoggingMixin", "SavingMixin", "Saving", "Grid",
+    "Sampler", "context_error_handler", "timeit", "lock_manager", "batch_manager", "timing_context",
+    "data_tuple_saving_controller", "nested_type", "all_nested_type", "union_nested_type",
+    "flattened_type", "all_flattened_type", "union_flattened_type", "Nested"
 ]
