@@ -9,30 +9,32 @@ from typing import *
 from scipy import interp
 from scipy import stats as ss
 from sklearn import metrics
-from itertools import product
+from functools import partial
 
 from ..misc import *
 
 
 class Anneal:
     """
-    Util class which can provide annealed numbers with given `method`
-    * Formulas could be found in `_initialize` method
+    Util class which can provide annealed numbers with given `method`.
+    * Formulas could be found in `_initialize` method.
 
     Parameters
     ----------
-    method : str, indicates which anneal method to be used
-    n_iter : int, indicates how much 'steps' will be taken to reach `ceiling` from `floor`
-    floor : float, indicates the start point of the annealed number
-    ceiling : float, indicates the end point of the annealed number
+    method : str, indicates which anneal method to be used.
+    n_iter : int, indicates how much 'steps' will be taken to reach `ceiling` from `floor`.
+    floor : float, indicates the start point of the annealed number.
+    ceiling : float, indicates the end point of the annealed number.
 
     Examples
     --------
+    >>> from cftool.ml.utils import Anneal
+    >>>
     >>> anneal = Anneal("linear", 50, 0.01, 0.99)
     >>> for i in range(100):
     >>>     # for i == 0, 1, ..., 48, 49, it will pop 0.01, 0.03, ..., 0.97, 0.99
     >>>     # for i == 50, 51, ..., 98, 99, it will pop 0.99, 0.99, ..., 0.99, 0.99
-    >>>     anneal.pop()
+    >>>     print(anneal.pop())
 
     """
 
@@ -90,26 +92,28 @@ class Anneal:
 
 class Metrics(LoggingMixin):
     """
-    Util class to calculate a whole variety of metrics
+    Util class to calculate a whole variety of metrics.
 
     Warnings
     ----------
-    * Notice that 2-dimensional arrays are desired, not flattened arrays
-    * Notice that first two args of each metric method must be `y` & `pred`
+    * Notice that 2-dimensional arrays are desired, not flattened arrays.
+    * Notice that first two args of each metric method must be `y` & `pred`.
 
     Parameters
     ----------
-    metric_type : str, indicates which kind of metric is to be calculated
-    config : dict
-        Configuration for the specific metric
-        * e.g. for quantile metric, you need to specify which quantile is to be evaluated
-    verbose_level : int, verbose level of Metrics
+    metric_type : str, indicates which kind of metric is to be calculated.
+    config : dict, configuration for the specific metric.
+    * e.g. For quantile metric, you need to specify which quantile is to be evaluated.
+    verbose_level : int, verbose level of `Metrics`.
 
     Examples
     --------
     >>> import numpy as np
+    >>>
+    >>> from cftool.ml.utils import Metrics
+    >>>
     >>> predictions, y_true = map(np.atleast_2d, [[1., 2., 3.], [0., 2., 1.]])
-    >>> print(Metrics("mae", {}).score(y_true.T, predictions.T))  # will be 1.
+    >>> print(Metrics("mae", {}).metric(y_true.T, predictions.T))  # will be 1.
 
     """
 
@@ -172,7 +176,7 @@ class Metrics(LoggingMixin):
         if requires_prob:
             cls.requires_prob_metrics.add(name)
 
-    def score(self, y, pred):
+    def metric(self, y, pred):
         if self.type is None:
             raise ValueError("`score` method was called but type is not specified in Metrics")
         y, pred = self._handle_nan(y, pred)
@@ -280,21 +284,30 @@ class Metrics(LoggingMixin):
         return thresholds[np.argmax(metric)]
 
 
-class Estimator:
+estimate_fn_type = Callable[[np.ndarray], np.ndarray]
+scoring_fn_type = Callable[[List[float], float, float], float]
+
+
+class Estimator(LoggingMixin):
     """
-    Util class to estimate the performances of a group of methods, on specific dataset & metric
+    Util class to estimate the performances of a group of methods, on specific dataset & metric.
 
     Parameters
     ----------
-    metric_type : str, indicates which kind of metric is to be calculated
-    **kwargs : used to initialize `Metrics` instance
+    metric_type : str, indicates which kind of metric is to be calculated.
+    verbose_level : int, verbose level used in `LoggingMixin`.
+    **kwargs : used to initialize `Metrics` instance.
 
     Examples
     --------
     >>> import numpy as np
+    >>>
+    >>> from cftool.ml.utils import Estimator
+    >>>
     >>> x, y = map(np.atleast_2d, [[1., 2., 3.], [0., 2., 1.]])
     >>> identical = lambda x_: x_
     >>> minus_one = lambda x_: x_ - 1
+    >>> # ~~~  [ info ] Results
     >>> # ==========================================================
     >>> # |             identical  |    mae     |  1.000000  |
     >>> # |             minus_one  |    mae     |  0.666667  |  <-
@@ -303,9 +316,13 @@ class Estimator:
 
     """
 
-    def __init__(self, metric_type: str, **kwargs):
-        self.scores = {}
-        self.best_method = None
+    def __init__(self,
+                 metric_type: str,
+                 *,
+                 verbose_level: int = 2,
+                 **kwargs):
+        self._reset()
+        self._verbose_level = verbose_level
         self._metric = Metrics(metric_type, **kwargs)
 
     def __str__(self):
@@ -317,58 +334,92 @@ class Estimator:
     def type(self) -> str:
         return self._metric.type
 
+    @property
+    def sign(self) -> int:
+        return self._metric.sign
+
+    @property
+    def requires_prob(self) -> bool:
+        return self._metric.requires_prob
+
+    # Core
+
+    def _reset(self):
+        self.raw_metrics = {}
+        self.final_scores = {}
+        self.best_method = None
+
+    def _default_scoring(self, raw_metrics, mean, std) -> float:
+        return mean - self.sign * std
+
+    # API
+
     def estimate(self,
                  x: np.ndarray,
                  y: np.ndarray,
-                 methods: Dict[str, Union[None, Callable]],
+                 methods: Dict[str, Union[estimate_fn_type, List[estimate_fn_type]]],
                  *,
-                 verbose: bool = True) -> None:
-        self.scores = {
-            name: None if method is None else self._metric.score(y, method(x))
-            for name, method in methods.items()
+                 scoring_function: Union[str, scoring_fn_type] = "default",
+                 verbose_level: int = 1) -> None:
+        self._reset()
+        if isinstance(scoring_function, str):
+            scoring_function = getattr(self, f"_{scoring_function}_scoring")
+        for k, v in methods.items():
+            if not isinstance(v, list):
+                methods[k] = [v]
+        self.raw_metrics = {
+            name: np.array([self._metric.metric(y, method(x)) for method in sub_methods], np.float32)
+            for name, sub_methods in methods.items()
         }
         msg_list = []
         best_idx, best_score = -1, -math.inf
-        sorted_method_names = sorted(self.scores)
+        sorted_method_names = sorted(self.raw_metrics)
         for i, name in enumerate(sorted_method_names):
-            score = self.scores[name]
-            if score is None:
-                continue
-            if verbose:
-                msg_list.append(f"|  {name:>20s}  |  {self._metric.type:^8s}  |  {score:8.6f}  |")
-            new_score = score * self._metric.sign
+            raw_metrics = self.raw_metrics[name]
+            mean, std = raw_metrics.mean().item(), raw_metrics.std().item()
+            msg_list.append(f"|  {name:>20s}  |  {self.type:^8s}  |  {mean:8.6f} ± {std:8.6f}  |")
+            new_score = scoring_function(raw_metrics, mean, std) * self.sign
+            self.final_scores[name] = new_score
             if new_score > best_score:
                 best_idx, best_score = i, new_score
         self.best_method = sorted_method_names[best_idx]
-        if verbose:
-            msg_list[best_idx] += "  <-  "
-            width = max(map(len, msg_list))
-            msg_list.insert(0, "=" * width)
-            msg_list.append("-" * width)
-            print("\n".join(msg_list))
+        msg_list[best_idx] += "  <-  "
+        width = max(map(len, msg_list))
+        msg_list.insert(0, "=" * width)
+        msg_list.append("-" * width)
+        self.log_block_msg("\n".join(msg_list), self.info_prefix, "Results", verbose_level)
 
 
 class ModelPattern(LoggingMixin):
     """
-    Util class to create an interface for users to leverage `Comparer` (and more in the future)
+    Util class to create an interface for users to leverage `Comparer` & `HPO` (and more in the future).
 
     Parameters
     ----------
     init_method : Callable[[], object]
-    * If None, then `ModelPattern` will not perform model creation
-    * If Callable, then `ModelPattern` will initialize a model with it
+    * If None, then `ModelPattern` will not perform model creation.
+    * If Callable, then `ModelPattern` will initialize a model with it.
     train_method : Callable[[object], None]
-    * If None, then `ModelPattern` will not perform model training
-    * If Callable, then `ModelPattern` will train the created model (from `init_method`) with it
+    * If None, then `ModelPattern` will not perform model training.
+    * If Callable, then `ModelPattern` will train the created model (from `init_method`) with it.
     predict_method : Union[str, Callable[[np.ndarray], np.ndarray]]
-    * If str, then `ModelPattern` will use `getattr` to get the predict method of the model obtained from above
-        In this case, `init_method` must be provided (`train_method` is still optional, because you can create a
-      trained model in `init_method`)
-    * If Callable, then `ModelPattern` will use it for prediction
+    * If str, then `ModelPattern` will use `getattr` to get the label predict method of the model obtained
+        from above. In this case, `init_method` must be provided (`train_method` is still optional, because you can
+        create a trained model in `init_method`).
+    * If Callable, then `ModelPattern` will use it for label prediction.
+    * Notice that predict_method should return a column vector (e.g. out.shape = [n, 1])
+    predict_prob_method : Union[str, Callable[[np.ndarray], np.ndarray]]
+    * If str, then `ModelPattern` will use `getattr` to get the probability prediction method of the model obtained
+        from above. In this case, `init_method` must be provided (`train_method` is still optional, because you can
+        create a trained model in `init_method`).
+    * If Callable, then `ModelPattern` will use it for probability prediction.
 
     Examples
     --------
     >>> import numpy as np
+    >>>
+    >>> from cftool.ml.utils import ModelPattern
+    >>>
     >>> x, y = map(np.atleast_2d, [[1., 2., 3.], [0., 2., 1.]])
     >>> predict_method = lambda x_: x_ - 1
     >>> init_method = lambda: type("Test", (), {"predict": lambda self, x_: predict_method(x_)})()
@@ -383,7 +434,7 @@ class ModelPattern(LoggingMixin):
                  init_method: Callable[[], object] = None,
                  train_method: Callable[[object], None] = None,
                  predict_method: Union[str, Callable[[np.ndarray], np.ndarray]] = "predict",
-                 trigger_logging: bool = False,
+                 predict_prob_method: Union[str, Callable[[np.ndarray], np.ndarray]] = "predict_prob",
                  verbose_level: int = 2):
         if init_method is None:
             self.model = None
@@ -392,11 +443,12 @@ class ModelPattern(LoggingMixin):
         if train_method is not None:
             train_method(self.model)
         self._predict_method = predict_method
-        self._init_logging(verbose_level, trigger_logging)
+        self._predict_prob_method = predict_prob_method
+        self._verbose_level = verbose_level
 
-    def predict(self,
-                x: np.ndarray) -> np.ndarray:
-        predict_method = self._predict_method
+    def predict_method(self,
+                       requires_prob: bool) -> Callable[[np.ndarray], np.ndarray]:
+        predict_method = self._predict_prob_method if requires_prob else self._predict_method
         if isinstance(predict_method, str):
             if self.model is None:
                 raise ValueError("Either init_method or Callable predict_method is required in ModelPattern")
@@ -406,25 +458,129 @@ class ModelPattern(LoggingMixin):
                 "predict_method is Callable but model is also created, which has no effect",
                 self.warning_prefix, 2, logging.WARNING
             )
-        return predict_method(x)
+        return predict_method
+
+    def predict(self,
+                x: np.ndarray,
+                *,
+                requires_prob: bool = False) -> np.ndarray:
+        return self.predict_method(requires_prob)(x)
+
+    @classmethod
+    def repeat(cls, n: int, **kwargs) -> List["ModelPattern"]:
+        return [cls(**kwargs) for _ in range(n)]
 
 
-class Comparer:
+collate_fn_type = Callable[[List[np.ndarray], bool], np.ndarray]
+
+
+class EnsemblePattern:
     """
-    Util class to compare a group of `ModelPattern`s on a group of `Estimator`s
+    Util class to create an interface for users to leverage `Comparer` & `HPO` in an ensembled way.
 
     Parameters
     ----------
-    model_patterns : Dict[str, Union[ModelPattern, Dict[str, ModelPattern]]]
-    * If values are `ModelPattern`, then all estimators will use this only `ModelPattern` make predictions
-    * If values are Dict[str, ModelPattern], then each estimator will use values.get(estimator.type) to
-      make predictions. If corresponding `ModelPattern` does not exist (values.get(estimator.type) is None),
-      then corresponding estimation will be skipped
-    estimators : List[Estimator], list of estimators which we are interested in
+    model_patterns : List[ModelPattern], list of `ModelPattern` we want to ensemble from.
+    ensemble_method : Union[str, collate_fn_type], ensemble method we use to collate the results.
+    * If str, then `EnsemblePattern` will use `getattr` to get the collate function.
+        Currently only 'default' is supported, which implements voting for classification
+        and averaging for regression.
+    * If collate_fn_type, then `EnsemblePattern` will use it to collate the results directly.
 
     Examples
     --------
     >>> import numpy as np
+    >>>
+    >>> from cftool.ml.utils import ModelPattern, EnsemblePattern
+    >>>
+    >>> x, y = map(np.atleast_2d, [[1., 2., 3.], [0., 2., 1.]])
+    >>> identical = lambda x_: x_
+    >>> minus_one = lambda x_: x_ - 1
+    >>> identical_pattern = ModelPattern(predict_method=identical)
+    >>> minus_one_pattern = ModelPattern(predict_method=minus_one)
+    >>> # Averaging 'identical' & 'minus_one' -> 'minus_0.5'
+    >>> ensemble = EnsemblePattern([identical_pattern, minus_one_pattern])
+    >>> # [[0.5 1.5 2.5]]
+    >>> print(ensemble.predict(x))
+
+    """
+
+    def __init__(self,
+                 model_patterns: List[ModelPattern],
+                 ensemble_method: Union[str, collate_fn_type] = "default"):
+        self._patterns = model_patterns
+        self._ensemble_method = ensemble_method
+
+    def __len__(self):
+        return len(self._patterns)
+
+    @property
+    def collate_fn(self) -> collate_fn_type:
+        if callable(self._ensemble_method):
+            return self._ensemble_method
+        return getattr(self, f"_{self._ensemble_method}_collate")
+
+    # Core
+
+    @staticmethod
+    def _default_collate(arrays: List[np.ndarray],
+                         requires_prob: bool) -> np.ndarray:
+        predictions = np.array(arrays)
+        if not requires_prob and np.issubdtype(predictions.dtype, np.integer):
+            max_class = predictions.max() + 1
+            predictions = predictions.squeeze(2).T
+            counts = np.apply_along_axis(partial(np.bincount, minlength=max_class), 1, predictions)
+            return counts.argmax(1).reshape([-1, 1])
+        return predictions.mean(0)
+
+    # API
+
+    def predict_method(self,
+                       requires_prob: bool) -> Callable[[np.ndarray], np.ndarray]:
+        predict_methods = list(map(ModelPattern.predict_method, self._patterns, len(self) * [requires_prob]))
+        def _predict(x: np.ndarray):
+            predictions = [method(x) for method in predict_methods]
+            return self.collate_fn(predictions, requires_prob)
+        return _predict
+
+    def predict(self,
+                x: np.ndarray,
+                *,
+                requires_prob: bool = False) -> np.ndarray:
+        return self.predict_method(requires_prob)(x)
+
+    @classmethod
+    def from_same_methods(cls,
+                          n: int,
+                          ensemble_method: Union[str, collate_fn_type] = "default",
+                          **kwargs):
+        return cls([ModelPattern(**kwargs) for _ in range(n)], ensemble_method)
+
+
+pattern_type = Union[ModelPattern, EnsemblePattern]
+patterns_type = Union[pattern_type, List[pattern_type]]
+
+
+class Comparer(LoggingMixin):
+    """
+    Util class to compare a group of `patterns_type`s on a group of `Estimator`s.
+
+    Parameters
+    ----------
+    patterns : Dict[str, Union[patterns_type, Dict[str, patterns_type]]]
+    * If values are `patterns_type`, then all estimators will use this only `patterns_type` make predictions.
+    * If values are Dict[str, patterns_type], then each estimator will use values.get(estimator.type) to
+      make predictions. If corresponding `patterns` does not exist (values.get(estimator.type) is None),
+      then corresponding estimation will be skipped.
+    estimators : List[Estimator], list of estimators which we are interested in.
+    verbose_level : int, verbose level used in `LoggingMixin`.
+
+    Examples
+    --------
+    >>> import numpy as np
+    >>>
+    >>> from cftool.ml.utils import ModelPattern, Estimator, Comparer
+    >>>
     >>> x, y = map(np.atleast_2d, [[1., 2., 3.], [0., 2., 1.]])
     >>> identical = lambda x_: x_
     >>> minus_one = lambda x_: x_ - 1
@@ -433,10 +589,12 @@ class Comparer:
     >>>     "minus_one": ModelPattern(predict_method=minus_one)
     >>> }
     >>> estimators = [Estimator("mse"), Estimator("mae")]
-    >>> #==========================================================
+    >>> # ~~~  [ info ] Results
+    >>> # ==========================================================
     >>> # |             identical  |    mse     |  1.666667  |
     >>> # |             minus_one  |    mse     |  0.666667  |  <-
     >>> # ----------------------------------------------------------
+    >>> # ~~~  [ info ] Results
     >>> # ==========================================================
     >>> # |             identical  |    mae     |  1.000000  |
     >>> # |             minus_one  |    mae     |  0.666667  |  <-
@@ -451,14 +609,21 @@ class Comparer:
     """
 
     def __init__(self,
-                 model_patterns: Dict[str, Union[ModelPattern, Dict[str, ModelPattern]]],
-                 estimators: List[Estimator]):
-        self.model_patterns = model_patterns
+                 patterns: Dict[str, Union[patterns_type, Dict[str, patterns_type]]],
+                 estimators: List[Estimator],
+                 *,
+                 verbose_level: int = 2):
+        self.patterns = patterns
         self.estimators = dict(zip([estimator.type for estimator in estimators], estimators))
+        self._verbose_level = verbose_level
 
     @property
-    def scores(self) -> Dict[str, Dict[str, Union[float, None]]]:
-        return {k: v.scores for k, v in self.estimators.items()}
+    def raw_metrics(self) -> Dict[str, Dict[str, np.ndarray]]:
+        return {k: v.raw_metrics for k, v in self.estimators.items()}
+
+    @property
+    def final_scores(self) -> Dict[str, Dict[str, float]]:
+        return {k: v.final_scores for k, v in self.estimators.items()}
 
     @property
     def best_methods(self) -> Dict[str, str]:
@@ -468,30 +633,57 @@ class Comparer:
                 x: np.ndarray,
                 y: np.ndarray,
                 *,
-                verbose: bool = True) -> "Comparer":
+                scoring_function: Union[str, scoring_fn_type] = "default",
+                verbose_level: int = 1) -> "Comparer":
         for estimator in self.estimators.values():
             methods = {}
-            for model_name, pattern in self.model_patterns.items():
-                if isinstance(pattern, dict):
-                    pattern = pattern.get(estimator.type)
-                if pattern is None:
+            for model_name, patterns in self.patterns.items():
+                if isinstance(patterns, dict):
+                    patterns = patterns.get(estimator.type)
+                if patterns is None:
                     continue
-                methods[model_name] = pattern.predict
-            estimator.estimate(x, y, methods, verbose=verbose)
+                if not isinstance(patterns, list):
+                    patterns = [patterns]
+                invalid = False
+                predict_methods = []
+                requires_prob = estimator.requires_prob
+                for pattern in patterns:
+                    if pattern is None:
+                        invalid = True
+                        break
+                    predict_methods.append(pattern.predict_method(requires_prob))
+                    if predict_methods[-1] is None:
+                        invalid = True
+                        self.log_msg(
+                            f"{estimator} requires probability predictions but {model_name} "
+                            f"does not have probability predicting method, skipping",
+                            self.warning_prefix, verbose_level, logging.WARNING
+                        )
+                        break
+                if invalid:
+                    continue
+                methods[model_name] = predict_methods
+            estimator.estimate(
+                x, y, methods,
+                scoring_function=scoring_function,
+                verbose_level=verbose_level
+            )
         return self
 
 
 class ScalarEMA:
     """
-    Util class to record Exponential Moving Average (EMA) for scalar value
+    Util class to record Exponential Moving Average (EMA) for scalar value.
 
     Parameters
     ----------
-    decay : float, decay rate for EMA
-        * Formula: new = (1 - decay) * current + decay * history; history = new
+    decay : float, decay rate for EMA.
+    * new = (1 - decay) * current + decay * history; history = new
 
     Examples
     --------
+    >>> from cftool.ml.utils import ScalarEMA
+    >>>
     >>> ema = ScalarEMA(0.5)
     >>> for i in range(4):
     >>>     print(ema.update("score", 0.5 ** i))  # 1, 0.75, 0.5, 0.3125
@@ -520,90 +712,52 @@ class ScalarEMA:
         return updated
 
 
-class Grid:
-    """
-    Util class provides permutation of simple, flattened param dicts
-    * For permutation of complex, nested param dicts, please refers to `ParamGenerator` in `cftool.param_utils.basis`
-
-    Parameters
-    ----------
-    param_grid : dict[str, list(int)]
-        Indicates param names and corresponding possible values
-
-    Examples
-    ----------
-    >>> grid = Grid({"a": [1, 2, 3], "b": [1, 2, 3]})
-    >>> for param in grid:
-    >>>     print(param)
-    >>> # output : {'a': 1, 'b': 1}, {'a': 1, 'b': 2}, {'a': 1, 'b': 3}
-    >>> #          {'a': 2, 'b': 1}, ..., {'a': 3, 'b': 3}
-
-    """
-
-    def __init__(self, param_grid):
-        self._grid = param_grid
-
-    def __iter__(self):
-        items = sorted(self._grid.items())
-        if not items:
-            yield {}
-        else:
-            keys, values = zip(*items)
-            for v in map(list, product(*values)):
-                for i, vv in enumerate(v):
-                    if isinstance(vv, tuple) and len(vv) == 1:
-                        v[i] = vv[0]
-                params = dict(zip(keys, v))
-                yield params
-
-
 class Visualizer:
     """
-    Visualization class
+    Visualization class.
 
     Methods
     ----------
     bar(self, data, classes, categories, save_name="bar_plot", title="",
             padding=1e-3, expand_floor=5, replace=True)
-        Make bar plot with given `data`
-        * data : np.ndarray
-            Containing values for the bar plot, where data.shape =
-            * (len(categories), ), if len(classes) == 1
-            * (len(classes), len(categories)), otherwise
-        * classes : list(str), list of str which indicates each class
-            * each class will has its own color
-            * len(classes) indicates how many bars are there in one category (side by side)
-        * categories : list(str), list of str which indicates each category
-            * a category will be a tick along x-axis
-        * save_name : str, saving name of this bar plot
-        * title : str, title of this bar plot
-        * padding : float, minimum value of each bar
-        * expand_floor : int, when len(categories) > `expand_floor`, the width of the figure will expand
-            * for len(classes) == 1, `expand_floor` will be multiplied by 2 internally
+        Make bar plot with given `data`.
+        * data : np.ndarray, containing values for the bar plot, where data.shape =
+            * (len(categories), ), if len(classes) == 1.
+            * (len(classes), len(categories)), otherwise.
+        * classes : list(str), list of str which indicates each class.
+            * each class will has its own color.
+            * len(classes) indicates how many bars are there in one category (side by side).
+        * categories : list(str), list of str which indicates each category.
+            * a category will be a tick along x-axis.
+        * save_name : str, saving name of this bar plot.
+        * title : str, title of this bar plot.
+        * padding : float, minimum value of each bar.
+        * expand_floor : int, when len(categories) > `expand_floor`, the width of the figure will expand.
+            * for len(classes) == 1, `expand_floor` will be multiplied by 2 internally.
         * overwrite : bool
-            whether overwrite the existing file with the same file name of this plot's saving name
+            whether overwrite the existing file with the same file name of this plot's saving name.
 
     function(self, f, x_min, x_max, classes, categories, save_names=None,
              n_sample=1000, expand_floor=5, overwrite=True):
         Make multiple (len(categories)) line plots with given function (`f`)
         * f : function
-            * input should be an np.ndarray with shape == (n, n_categories)
-            * output should be an np.ndarray with shape == (n, n_categories, n_categories)
-        * x_min : np.ndarray, minimum x-values for each line plot
-            * len(x_min) should be len(categories)
-        * x_max : np.ndarray, maximum x-values for each line plot
-            * len(x_max) should be len(categories)
-        * classes : list(str), list of str which indicates each class
-            * each class will has its own color
-            * len(classes) indicates how many bars are there in one category (side by side)
-        * categories : list(str), list of str which indicates each category
-            * every category will correspond to a line plot
-        * save_names : list(str), saving names of these line plots
-        * n_sample : int, sample density along x-axis
+            * input should be an np.ndarray with shape == (n, n_categories).
+            * output should be an np.ndarray with shape == (n, n_categories, n_categories).
+        * x_min : np.ndarray, minimum x-values for each line plot.
+            * len(x_min) should be len(categories).
+        * x_max : np.ndarray, maximum x-values for each line plot.
+            * len(x_max) should be len(categories).
+        * classes : list(str), list of str which indicates each class.
+            * each class will has its own color.
+            * len(classes) indicates how many bars are there in one category (side by side).
+        * categories : list(str), list of str which indicates each category.
+            * every category will correspond to a line plot.
+        * save_names : list(str), saving names of these line plots.
+        * n_sample : int, sample density along x-axis.
         * expand_floor : int, the width of the figures will be expanded with ratios calculated by:
             expand_ratios = np.maximum(1., np.abs(x_min) / expand_floor, x_max / expand_floor)
         * overwrite : bool
-            whether overwrite the existing file with the same file name of this plot's saving name
+            whether overwrite the existing file with the same file name of this plot's saving name.
 
     """
 
@@ -753,6 +907,7 @@ class Visualizer:
 
 
 __all__ = [
-    "Anneal", "Metrics", "ScalarEMA", "Grid", "Visualizer",
-    "Estimator", "ModelPattern", "Comparer"
+    "Anneal", "Metrics", "ScalarEMA", "Visualizer",
+    "Estimator", "ModelPattern", "EnsemblePattern", "Comparer",
+    "pattern_type", "patterns_type", "estimate_fn_type", "scoring_fn_type"
 ]
